/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "Clipboard.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomClipboard.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/PrivateGdomClipboard.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateIntGdomClipboard.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::Clipboard* obj)
{
    return WebCore::cacheDOMObject<WebCore::Clipboard, GDOMClipboard>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMClipboard::GDOMClipboard(PassRefPtr<WebCore::Clipboard> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMClipboard::~GDOMClipboard()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}


/* XXX TODO: custom function gdom_clipboard_clear_data */


/* XXX TODO: custom function gdom_clipboard_get_data */


/* XXX TODO: custom function gdom_clipboard_set_data */


/* XXX TODO: custom function gdom_clipboard_set_drag_image */

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "Clipboard.h"
#include <gdom/GdomClipboard.h>
#include <gdom/PrivateGdomClipboard.h>
#include <gdom/PrivateIntGdomClipboard.h>
#include <gdom/GdomClipboardprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomClipboard, gdom_clipboard, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomClipboardPrivate {
    class WebCore::Clipboard *item;
};

#define GDOM_CLIPBOARD_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_CLIPBOARD, GdomClipboardPrivate))
*/
G_END_DECLS
WebCore::Clipboard *
gdom_clipboard_get_private(GdomClipboard *request)
{
    GDOMClipboard *res;
    res = static_cast<GDOMClipboard*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::Clipboard*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_DROP_EFFECT,
    PROP_EFFECT_ALLOWED,
};


static void gdom_clipboard_finalize(GObject* object)
{
    /*GdomClipboard* request = GDOM_CLIPBOARD(object);*/
    /*GdomClipboardPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_clipboard_parent_class)->finalize(object);
}

static void gdom_clipboard__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomClipboard *self = GDOM_CLIPBOARD(object);
    WebCore::Clipboard * gself = (gdom_clipboard_get_private(self));
    switch (prop_id) {
     case PROP_DROP_EFFECT:
    {
         gself->setDropEffect(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_EFFECT_ALLOWED:
    {
         gself->setEffectAllowed(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_clipboard__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomClipboard *self = GDOM_CLIPBOARD(object);
    WebCore::Clipboard * gself = (gdom_clipboard_get_private(self));
    switch (prop_id) {
    case PROP_DROP_EFFECT:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->dropEffect()));
         break;
    }
    case PROP_EFFECT_ALLOWED:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->effectAllowed()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_clipboard_class_init(GdomClipboardClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_clipboard_finalize;
    g_klass->set_property = gdom_clipboard__set_property_fn;
    g_klass->get_property = gdom_clipboard__get_property_fn;

    g_object_class_install_property (g_klass, PROP_DROP_EFFECT,
            g_param_spec_string ("drop_effect", /* name */
                "clipboard_drop_effect", /* short description */
                "read-write  gchar * Clipboard.drop_effect", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_EFFECT_ALLOWED,
            g_param_spec_string ("effect_allowed", /* name */
                "clipboard_effect_allowed", /* short description */
                "read-write  gchar * Clipboard.effect_allowed", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomClipboardPrivate));*/
}

static void gdom_clipboard_init(GdomClipboard* request)
{
    /*GdomClipboardPrivate* priv = GDOM_CLIPBOARD_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

